/*
 *  DNSpoison
 *  src/dns.h
 *
 *  Copyright (C) 2011
 *
 *  Author: Vilmain Nicolas
 *  Contact: null.sim@gmail.com
 *
 *  This file is part of DNSpoison.
 *
 *  DNSpoison is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DNSpoison is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DNSpoison.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DNSRP_DNS_H__
#define __DNSRP_DNS_H__

#include  "ostype.h"

#define    DNS_RR_SIZE      261
#define    DNS_SIZE         sizeof (struct dnshdr_s)
#define    DNS_PACKET       346

struct dnshdr_s
  {
    uint16_t id;
#ifdef __IS_LITTLE_ENDIAN
    uint8_t rd :1,
            tc :1,
            aa :1,
            opcode :4,
            orr :1;
    uint8_t rcode :4,
            z :3,
            ra :1;
#elif defined(__IS_BIG_ENDIAN)
    uint8_t orr :1,
            opcode :4,
            aa :1,
            tc :1,
            rd :1;
    uint8_t ra :1,
            z :3,
            rcode :4;
#endif /* not __PROC_IS_LITTLE_ENDIAN */
    uint16_t qdcount;
    uint16_t ancount;
    uint16_t nscount;
    uint16_t arcount;
    char rr[DNS_RR_SIZE];
  };

struct dns_rr_s
  {
    uint16_t type;
    uint16_t class;
    uint32_t ttl;
    uint16_t len;
    union
      {
	uint8_t ipv4[4];
	uint8_t ipv6[16];
      } un;
  }__attribute__((packed));

struct dns_rr_ans_s
  {
    uint16_t host;
    struct dns_rr_s rr;
  } __attribute__((packed));

#endif /* not __DNSRP_DNS_H__ */
