/*
 *  DNSpoison
 *  src/dnspoison.h
 *
 *  Copyright (C) 2011
 *
 *  Author: Vilmain Nicolas
 *  Contact: null.sim@gmail.com
 *
 *  This file is part of DNSpoison.
 *
 *  DNSpoison is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DNSpoison is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DNSpoison.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DNS_REQUEST_POISON_H__
#define __DNS_REQUEST_POISON_H__

#include  "ostype.h"

#if !defined(__BSD) && !defined(__Linux)
# error "SET MACROS __Linux or __BSD"
#endif /* not __Linux and not __BSD */

#if !defined(__IS_LITTLE_ENDIAN) && !defined(__IS_BIG_ENDIAN)
# error "UNKNOW PROC TYPE (BIG OR LITTLE ENDIAN)"
#endif /* not __IS_LITTLE_ENDIAN and not __IS_BIG_ENDIAN */

#include  <time.h>
#include  <fcntl.h>
#include  <errno.h>
#include  <stdio.h>
#include  <stdlib.h>
#include  <stdint.h>
#include  <string.h>
#include  <unistd.h>
#include  <stdarg.h>
#include  <signal.h>
#include  <sys/time.h>
#include  <sys/ioctl.h>
#include  <netinet/in.h>
#include  <arpa/inet.h>
#include  <sys/socket.h>
#ifdef __Linux
# include  <linux/if.h>
# include  <linux/if_ether.h>
# include  <linux/if_packet.h>
#elif defined (__BSD)
# include  <net/if.h>
# include  <net/if_dl.h>
# include  <net/bpf.h>
# include  <net/ethernet.h>
#endif /* not __Linux */
#include  <ifaddrs.h>
#include  "dns.h"

#define PROGRAM_VERSION  "DNS poison 1.0"

#ifdef __Linux
# define  ETH_IP      ETH_P_IP
#elif defined(__BSD)
# define  ETH_IP      ETHERTYPE_IP
#endif /* not __Linux */
#define  PROTO_UDP    IPPROTO_UDP

#define  MAC_SIZE     6
#define  NAME_LEN     17
#define  IP4_SIZE     4
#define  IP6_SIZE     sizeof (struct in6_addr)
#define  URL_LEN      255
#define  PACKET_SIZE  65535

#define  COPY_MAC(dst, src)    memcpy (dst, src, MAC_SIZE)
#define  CMP_MAC(mac1, mac2)   memcmp (mac1, mac2, MAC_SIZE)

#define  NULL_IPv4             "\x00\x00\x00\x00"
#define  NULL_IPv6             "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
#define  COPY_IP(dst, src)     memcpy (dst, src, IP4_SIZE)
#define  CMP_IP(ip1, ip2)      memcmp (ip1, ip2, IP4_SIZE)
#define  COPY_IPv6(dst, src)   memcpy (dst, src, IP6_SIZE)
#define  CMP_IPv6(ip1, ip2)    memcmp (ip1, ip2, IP6_SIZE)


#define  MIN_PACKET				\
  (sizeof (struct ethhdr_s)			\
   + sizeof (struct iphdr_s)			\
   + sizeof (struct udphdr_s))

struct netconf_s
  {
    int nc_index;
    int nc_mtu;
    char nc_name[NAME_LEN];
    struct in_addr nc_ipv4;
  };

struct ethsock_s
  {
    int fdsock;
#ifdef __Linux
    struct sockaddr_ll ll;
# define   SOCK_LL_SIZE   sizeof (struct sockaddr_ll)
#endif /* __Linux */
  };

struct url_s
  {
    char *url;
    struct in_addr **ipv4;
    struct in6_addr **ipv6;
    int n_ip4;
    int n_ip6;
    struct url_s *prev;
  };

struct ethhdr_s
  {
    uint8_t dst[MAC_SIZE];
    uint8_t src[MAC_SIZE];
    uint16_t protocol;
  };

struct iphdr_s
  {
#ifdef __IS_LITTLE_ENDIAN
    uint8_t ihl:4,
            version:4;
#elif defined (__IS_BIG_ENDIAN)
    uint8_t version:4,
            ihl:4;
#else
#error  "unknow proc architecture"
#endif
    uint8_t tos;
    uint16_t tot_len;
    uint16_t id;
    uint16_t frag_off;
    uint8_t ttl;
    uint8_t protocol;
    uint16_t check;
    uint32_t saddr;
    uint32_t daddr;
  } __attribute__((packed));

struct udphdr_s
  {
    uint16_t source;
    uint16_t dest;
    uint16_t len;
    uint16_t check;
  };

struct psdhdr_s
  {
    unsigned long saddr;
    unsigned long daddr;
    char useless;
    unsigned char protocol;
    unsigned short length;
  };

#define DEFAULT_PKT_SIZE 1500
union packet
  {
    char packet[DEFAULT_PKT_SIZE];
    struct hdr
      {
	struct ethhdr_s eth;
	struct iphdr_s ip;
	struct udphdr_s udp;
      } hdr;
  };

void *  xcalloc (size_t size);
void *  xrealloc (void *ptr, size_t size);
char *  xstrdup (const char *str);
void    free_tab (void **tab);
unsigned short  cksum(unsigned short * addr,int len);
int     get_netconf (struct netconf_s *nc, const char *name);
int     get_ethsock (struct ethsock_s *s, int index, const char *name);
int     ethrecv (int fdsock, union packet *pkt, int max_size);
int     ethsend (struct ethsock_s *s, void *packet, size_t size);
int     packet_is_dns_request (union packet *pkt, char *url);
void    dns_decode_url (char *url);
int     make_dns_header (union packet *pkt, struct url_s *url,
			 int pkt_size, int ipvn);

char *program_name;

#endif /* not __DNS_REQUEST_POISON_H__ */
