/*
 *  DNSpoison
 *  src/utils.c
 *
 *  Copyright (C) 2011
 *
 *  Author: Vilmain Nicolas
 *  Contact: null.sim@gmail.com
 *
 *  This file is part of DNSpoison.
 *
 *  DNSpoison is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  DNSpoison is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with DNSpoison.  If not, see <http://www.gnu.org/licenses/>.
 */

#include  "dnspoison.h"

static void  out_memory (const char *err);

static void
out_memory (const char *err)
{
  fprintf (stderr, "%s:%s: memory exhausted\n",
	   program_name, err);
  exit (EXIT_FAILURE);
}

void *
xcalloc (size_t size)
{
  void *ptr = NULL;

  ptr = malloc (size);
  if (!ptr)
    out_memory ("malloc");
  memset (ptr, 0, size);
  return ptr;
}

void *
xrealloc (void *ptr, size_t size)
{
  if (!ptr)
    return xcalloc (size);
  ptr = realloc (ptr, size);
  if (!ptr)
    out_memory ("realloc");
  return ptr;
}

char *
xstrdup (const char *str)
{
  char *copy = NULL;

  copy = strdup (str);
  if (!copy)
    out_memory ("strdup");
  return copy;
}

void
free_tab (void **tab)
{
  int i;

  if (tab)
    {
      for (i = 0; tab[i]; i++)
	free (tab[i]);
      free (tab);
    }
}

/* this function is not coded by me */
unsigned short
cksum(unsigned short * addr,int len)
{
  register int sum = 0;
  unsigned short answer = 0;
  register unsigned short * w = addr;
  register int nleft = len;

  while(nleft > 1)
    {
      sum += *w++;
      nleft -= 2;
    }
  if(nleft == 1)
    {
      *(unsigned char *)(&answer) = *(unsigned char *) w;
      sum += answer;
    }
  sum = (sum >> 16) + (sum & 0xffff);
  sum += (sum >> 16);
  answer = ~sum;
  return answer;
}
/* end comments */
